﻿/**
 * seekTemplate - 前端轻量级模板插件
 * Created by likaituan on 14/8/18.
 */

~function (exp) {

	//根据指定的分隔符截成两半
	let split2 = function (str, flag) {
		let a = str.split(flag);
		let ret = [a.shift()];
		a.length > 0 && ret.push(a.join(flag));
		return ret;
	};

	//生成JS代码
	exp.getJsCode = function (tmpCode) {
		tmpCode = tmpCode.replace(/\r\n|\r|\n/g, ''); //去除换行符
		let jsCode = [];
		jsCode.push('let buf = [];');
		let R = RegExp;
		let literalRe = /\{literal\}([\s\S]+?)\{\/literal\}/;
		let jsRe = /<%([\s\S]+?)%>/;
		let smartyRe = /\{(.+?)\}/;

		//添加HTML代码
		let addHTML = function (html) {
			html = html.replace(/【/g, '{').replace(/】/g, '}');
			html = html.replace(/'/g, `\\'`);
			jsCode.push(`buf.push('${html}');`);
		};

		//添加JS代码
		let addJs = function (js) {
			js = js.replace(/\$\((.+?)\);?/g, "buf.push($1);");
			jsCode.push(js);
		};

		//解析Smarty代码
		let cache = {};
		let parseSmarty = function (syntax) {
			//syntax = syntax.replace(/\$/g, "this.");
			let parts = syntax.split(' ');
			let p1 = parts.shift();
			let [p2, p3, p4, p5, p6] = parts;
			if (p1 === 'for') {
				if (/^(?:from|in|of)$/.test(p3) === false) {
					throw 'for syntax only allow [from,in,of]';
				}
				if (p3 === 'from' && p5 === 'to') {
					let index = p2;
					let start = p4;
					let end = p6;
					jsCode.push(`for (let ${index} = ${start}; ${index} <= ${end}; ${index}++){`);
					return;
				}
				p2 = p2.split(',');
				let index = p3 === 'in' ? p2[0] : p2[1];
				let item = p3 === 'of' ? p2[0] : p2[1];
				let arr = cache.src = p4;
				if (index) {
					jsCode.push(`for (let ${index} in ${arr}){`);
					item && jsCode.push(`let ${item} = ${arr}[${index}];`);
					return;
				}
				jsCode.push(`for (let ${item} of ${arr}){`);
			}
			else if (p1 === 'forEmpty') {
				jsCode.push(`}; if (${cache.src}.length === 0) {`);
			}
			else if (p1 === '/for') {
				jsCode.push('}');
			}
			else if (p1 === 'if') {
				jsCode.push(`if (${parts.join(' ')}) {`);
			}
			else if (p1 === 'elsif') {
				jsCode.push(`} else if (${p2}) {`);
			}
			else if (p1 === 'else') {
				jsCode.push('} else {');
			}
			else if (p1 === '/if') {
				jsCode.push('}');
			}
			else if (syntax[0] === '=') {
				jsCode.push(`console.log(${syntax.slice(1)});`);
			}
			else {
				let [val, format] = split2(syntax, '|');
				//避免碰到||的时候
				if (format && format[0] == '|') {
					val = syntax;
					format = null;
				}
				val = val.trim();
				if (format) {
					let {method, args} = split2(format, ':');
					method = method.trim();
					args = (args || '').split(',').map(x => x.trim());
					args.unshift(val);
					val = `$.${method}.apply($, ${args})`;
				}
				val && jsCode.push(`buf.push(${val});`);
			}
		};

		//先编译literal字面量
		let compileLiteral = function (code) {
			if (literalRe.test(code)) {
				let a = [R.leftContext, R.$1.trim(), R.rightContext];
				a[0] && compileJs(a[0]);
				a[1] && addHTML(a[1]);
				a[2] && compileLiteral(a[2]);
			} else {
				compileJs(code);
			}
		};

		//然后编译JS
		let compileJs = function (code) {
			if (jsRe.test(code)) {
				let a = [R.leftContext, R.$1, R.rightContext];
				a[0] && compileSmarty(a[0]);
				a[1] && addJs(a[1]);
				a[2] && compileJs(a[2]);
			} else {
				compileSmarty(code);
			}
		};


		//最后编译Smarty语法
		let compileSmarty = function (code) {
			code = code.replace(/\{\{/g, '【').replace(/\}\}/g, '】');
			if (smartyRe.test(code)) {
				let a = [R.leftContext, R.$1, R.rightContext];
				a[0] && addHTML(a[0]);
				a[1] && parseSmarty(a[1]);
				a[2] && compileSmarty(a[2]);
			} else {
				addHTML(code);
			}
		};

		compileLiteral(tmpCode);

		jsCode.push(`return buf.join('');`);
		jsCode = jsCode.join('\n');
		window.showTemplate && console.log(jsCode);
		return jsCode;
	};

	//获得编译函数
	exp.getFun = function (tplCode) {
		let jsCode = exp.getJsCode(tplCode);
		return new Function('$', jsCode);
	};

	//编译函数
	exp.compileFun = function (tplFun) {
		return function (data) {
			return tplFun.call(data, $);
		};
	};

	//直接编译模板代码
	exp.compile = function (tplCode) {
		let jsCode = exp.getJsCode(tplCode);
		window.showTemplate && console.log(jsCode);
		let templateFun = new Function('$', jsCode);
		return exp.compileFun(templateFun);
	};


}(typeof module === 'object' ? module.exports : window.seekTemplate={});